@@ -421,13 +421,20 @@ the layout to the selected layout-name."
   (interactive
    (let* ((completion-ignore-case t)
 	  (target
-	   (riece-completing-read-identity
-	    "Channel/User: " riece-current-channels))
+	   (if riece-join-channel-candidate
+	       (let ((default (riece-format-identity
+			       riece-join-channel-candidate)))
+		 (riece-completing-read-identity
+		  (format "Channel/User (default %s): " default)
+		  riece-current-channels nil nil nil nil default))
+	     (riece-completing-read-identity
+	      "Channel/User: " riece-current-channels)))
 	  key)
      (if (and current-prefix-arg
-	      (riece-channel-p target))
+	      (riece-channel-p (riece-identity-prefix target)))
 	 (setq key
-	       (riece-read-passwd (format "Key for %s: " target))))
+	       (riece-read-passwd (format "Key for %s: "
+					  (riece-format-identity target)))))
      (list target key)))
   (let ((pointer (riece-identity-member target riece-current-channels)))
     (if pointer
@@ -454,8 +461,10 @@ the layout to the selected layout-name."
    (let* ((completion-ignore-case t)
 	 (target
 	  (riece-completing-read-identity
-	   "Channel/User: " riece-current-channels nil nil
-	   (cons (riece-format-identity riece-current-channel) 0)))
+	   (format "Channel/User (default %s): "
+		   (riece-format-identity riece-current-channel))
+	   riece-current-channels nil nil nil nil
+	   (riece-format-identity riece-current-channel)))
 	 message)
      (if (and current-prefix-arg
 	      (riece-channel-p (riece-identity-prefix target)))
