@@ -125,29 +125,36 @@ the `riece-server-keyword-map' variable."
     (riece-process-send-string process string)))
 
 (defun riece-open-server (server server-name)
-  (if (equal server-name "")
-      (message "Connecting to IRC server...")
-    (message "Connecting to %s..." server-name))
   (riece-server-keyword-bind server
-    (let* (selective-display
-	   (coding-system-for-read 'binary)
-	   (coding-system-for-write 'binary)
-	   (process
+    (let (selective-display
+	  (coding-system-for-read 'binary)
+	  (coding-system-for-write 'binary)
+	  process)
+      (if (equal server-name "")
+	  (message "Connecting to IRC server...")
+	(message "Connecting to %s..." server-name))
+      (setq process
 	    (funcall function (riece-server-process-name server-name)
 		     (concat " *IRC*" server-name)
-		     host service)))
+		     host service))
+      (if (equal server-name "")
+	  (message "Connecting to IRC server...done")
+	(message "Connecting to %s...done" server-name))
       (riece-reset-process-buffer process)
       (with-current-buffer (process-buffer process)
 	(setq riece-server-name server-name))
       (set-process-sentinel process 'riece-sentinel)
       (set-process-filter process 'riece-filter)
-      (if (or password
-	      riece-reconnect-with-password)
+      (if (equal server-name "")
+	  (message "Logging in to IRC server...")
+	(message "Logging in to %s..." server-name))
+      (if riece-reconnect-with-password	;password incorrect or not set.
+	  (unwind-protect
+	      (setq password (riece-read-passwd "Password: "))
+	    (setq riece-reconnect-with-password nil)))
+      (if password
 	  (riece-process-send-string process
-				     (format "PASS %s\r\n"
-					     (or password
-						 (riece-read-passwd
-						  "Password: ")))))
+				     (format "PASS %s\r\n" password)))
       (riece-process-send-string process
 				 (format "USER %s * * :%s\r\n"
 					 (user-real-login-name)
@@ -159,10 +166,7 @@ the `riece-server-keyword-map' variable."
 	      riece-nick-accepted 'sent
 	      riece-coding-system coding))
       (setq riece-process-list
-	    (cons process riece-process-list))))
-  (if (equal server-name "")
-      (message "Connecting to IRC server...done")
-    (message "Connecting to %s...done" server-name)))
+	    (cons process riece-process-list)))))
 
 (defun riece-reset-process-buffer (process)
   (save-excursion
