@@ -258,10 +258,23 @@ If optional argument CONFIRM is non-nil, ask which IRC server to connect."
   (message "%s" (substitute-command-keys "Type \\[describe-mode] for help")))
 
 (defun riece-exit ()
-  (setq riece-server nil)
   (if riece-save-variables-are-dirty
       (riece-save-variables-files))
-  (riece-clear-system)
+  (while riece-buffer-list
+    (if (and (get-buffer (car riece-buffer-list))
+	     (buffer-live-p (car riece-buffer-list)))
+	(funcall riece-buffer-dispose-function (car riece-buffer-list)))
+    (setq riece-buffer-list (cdr riece-buffer-list)))
+  (setq riece-server nil
+	riece-current-channels nil
+	riece-current-channel nil
+	riece-user-indicator nil
+	riece-channel-indicator "None"
+	riece-channel-list-indicator "No channel"
+	riece-away-indicator "-"
+	riece-operator-indicator "-"
+	riece-freeze-indicator "-")
+  (delete-other-windows)
   (run-hooks 'riece-exit-hook))
 
 (defun riece-command-mode ()
@@ -456,9 +469,12 @@ Instead, these commands are available:
       (setq queue (cdr queue)))
     (if dependencies
 	(error "Circular add-on dependency found"))
+    (setq addons (nreverse addons))
     (while addons
       (require (car addons))		;implicit dependency
       (funcall (intern (concat (symbol-name (car addons)) "-insinuate")))
+      (if riece-debug
+	  (message "Add-on %S is loaded" (car addons)))
       (setq addons (cdr addons)))))
 
 (provide 'riece)
